/*
 * $Id: messagedialog.cpp,v 1.1 2007/03/04 12:53:16 ozawa Exp $
 *
 * Copyright 2007- ONGS Inc. All rights reserved.
 * 
 * author: Masanori OZAWA (ozawa@ongs.co.jp)
 * version: $Revision: 1.1 $
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY ONGS INC ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL ONGS INC OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * The views and conclusions contained in the software and documentation are
 * those of the authors and should not be interpreted as representing official
 * policies, either expressed or implied, of the ONGS Inc.
 * 
 */

#include "../include/fusenshi.h"
#include "messagedialog.h"

static char* sprintfMessage(const Glib::ustring& msg, va_list args)
{
    char* result = NULL;
    vasprintf(&result, msg.c_str(), args);
    
    return result;
}

static Glib::StaticMutex g_NotifyThreadMutex = GLIBMM_STATIC_MUTEX_INIT;

static void waitNotifyFunc(bool &end)
{
	bool loop = true;

	gdk_threads_leave();
    while (loop) {
		gdk_threads_enter();
	    while (Gtk::Main::events_pending()) {
	        Gtk::Main::iteration();
	    }
		gdk_threads_leave();
	    
	    usleep(25 * 1000 /* 25ms */);
	    
	    g_NotifyThreadMutex.lock();
	    loop = !end;
	    g_NotifyThreadMutex.unlock();
    }
	gdk_threads_enter();

    while (Gtk::Main::events_pending()) {
        Gtk::Main::iteration();
    }
}

static void notifyVoidFunc(const sigc::slot<void> &slot, bool *pEnd)
{
    usleep(100 * 1000 /* 100ms */);

	slot();
	
	g_NotifyThreadMutex.lock();
	*pEnd = true;
	g_NotifyThreadMutex.unlock();
}

static void notifyBoolFunc(const sigc::slot<bool> &slot, bool *pEnd, bool *pRet)
{
    usleep(100 * 1000 /* 100ms */);

	*pRet = slot();
	
	g_NotifyThreadMutex.lock();
	*pEnd = true;
	g_NotifyThreadMutex.unlock();
}

static void notifyIntFunc(const sigc::slot<int> &slot, bool *pEnd, int *pRet)
{
    usleep(100 * 1000 /* 100ms */);

	*pRet = slot();
	
	g_NotifyThreadMutex.lock();
	*pEnd = true;
	g_NotifyThreadMutex.unlock();
}

static void notifyPointerFunc(const sigc::slot<void*> &slot, bool *pEnd, void **pRet)
{
    usleep(100 * 1000 /* 100ms */);

	*pRet = slot();
	
	g_NotifyThreadMutex.lock();
	*pEnd = true;
	g_NotifyThreadMutex.unlock();
}

namespace Gtk {
class NotifyDialog : public Gtk::MessageDialog
{
public:
	NotifyDialog(Window& parent, const Glib::ustring& message,
		bool use_markup=false, MessageType type=MESSAGE_INFO,
		ButtonsType buttons=BUTTONS_OK, bool modal=false);
	NotifyDialog(const Glib::ustring& message, bool use_markup=false,
		MessageType type=MESSAGE_INFO, ButtonsType buttons=BUTTONS_OK,
		bool modal=false);
	virtual ~NotifyDialog();

protected:
	virtual bool on_delete_event(GdkEventAny *pEvent);
};

NotifyDialog::NotifyDialog(Window& parent, const Glib::ustring& message,
	bool use_markup, MessageType type, ButtonsType buttons, bool modal) :
	Gtk::MessageDialog(parent, message, use_markup, type, buttons, modal)
{
}

NotifyDialog::NotifyDialog(const Glib::ustring& message, bool use_markup,
	MessageType type, ButtonsType buttons, bool modal) :
	Gtk::MessageDialog(message, use_markup, type, buttons, modal)
{
}

NotifyDialog::~NotifyDialog()
{
}

bool NotifyDialog::on_delete_event(GdkEventAny *pEvent)
{
	// ɥĤʤ
	return true;
}

}; /* namespace GTK */


//_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
MessageDialog::MessageDialog()
{
}

MessageDialog::~MessageDialog()
{
}

//_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
// Error message
void MessageDialog::messageError(const Glib::ustring& msg, bool modal, ...)
{
    va_list ap;
    va_start(ap, modal);
    char *pMsg = sprintfMessage(msg, ap);
    va_end(ap);
    
    messageErrorCore(pMsg, modal, NULL);

    free(pMsg);
}

void MessageDialog::messageError(const Glib::ustring& msg, bool modal, Gtk::Window *pParent, ...)
{
    va_list ap;
    va_start(ap, pParent);
    char *pMsg = sprintfMessage(msg, ap);
    va_end(ap);

    messageErrorCore(pMsg, modal, pParent);

    free(pMsg);
}

void MessageDialog::messageErrorCore(const Glib::ustring& msg, bool modal, Gtk::Window *pParent)
{
    Gtk::MessageDialog dlg(msg, true, Gtk::MESSAGE_ERROR, Gtk::BUTTONS_OK, modal);
    dlg.set_title(_("Error"));
    if (pParent) {
	    dlg.set_transient_for(*pParent);
	    dlg.set_position(Gtk::WIN_POS_CENTER_ON_PARENT);
    }
    else
	    dlg.set_position(Gtk::WIN_POS_CENTER);
    dlg.run();
}

//_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
// Warning message
void MessageDialog::messageWarning(const Glib::ustring& msg, bool modal, ...)
{
    va_list ap;
    va_start(ap, modal);
    char *pMsg = sprintfMessage(msg, ap);
    va_end(ap);
    
    messageWarningCore(pMsg, modal, NULL);
    
    free(pMsg);
}

void MessageDialog::messageWarning(const Glib::ustring& msg, bool modal, Gtk::Window *pParent, ...)
{
    va_list ap;
    va_start(ap, pParent);
    char *pMsg = sprintfMessage(msg, ap);
    va_end(ap);
    
    messageWarningCore(pMsg, modal, pParent);
    
    free(pMsg);
}

void MessageDialog::messageWarningCore(const Glib::ustring& msg, bool modal, Gtk::Window *pParent)
{
    Gtk::MessageDialog dlg(msg, true, Gtk::MESSAGE_WARNING, Gtk::BUTTONS_OK, modal);
    dlg.set_title(_("Warning"));
    if (pParent) {
	    dlg.set_transient_for(*pParent);
	    dlg.set_position(Gtk::WIN_POS_CENTER_ON_PARENT);
    }
    else
	    dlg.set_position(Gtk::WIN_POS_CENTER);
    dlg.run();
}

//_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
// Info message
void MessageDialog::messageInfo(const Glib::ustring& msg, bool modal, ...)
{
    va_list ap;
    va_start(ap, modal);
    char *pMsg = sprintfMessage(msg, ap);
    va_end(ap);
    
    messageInfoCore(pMsg, modal, NULL);
    
    free(pMsg);
}

void MessageDialog::messageInfo(const Glib::ustring& msg, bool modal, Gtk::Window *pParent, ...)
{
    va_list ap;
    va_start(ap, pParent);
    char *pMsg = sprintfMessage(msg, ap);
    va_end(ap);
    
    messageInfoCore(pMsg, modal, pParent);
    
    free(pMsg);
}

void MessageDialog::messageInfoCore(const Glib::ustring& msg, bool modal, Gtk::Window *pParent)
{
    Gtk::MessageDialog dlg(msg, true, Gtk::MESSAGE_INFO, Gtk::BUTTONS_OK, modal);
    dlg.set_title(_("Information"));
    if (pParent) {
	    dlg.set_transient_for(*pParent);
	    dlg.set_position(Gtk::WIN_POS_CENTER_ON_PARENT);
    }
    else
	    dlg.set_position(Gtk::WIN_POS_CENTER);
    dlg.run();
}

//_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
// Question message
int MessageDialog::messageQuestion(const Glib::ustring& msg, bool modal,
	Gtk::ButtonsType buttonType, ...)
{
    va_list ap;
    va_start(ap, buttonType);
    char *pMsg = sprintfMessage(msg, ap);
    va_end(ap);
    
    int result = messageQuestionCore(pMsg, modal, buttonType, 0, NULL);
    
    free(pMsg);
    
    return result;
}

int MessageDialog::messageQuestion(const Glib::ustring& msg, bool modal,
	Gtk::ButtonsType buttonType, Gtk::Window *pParent, ...)
{
    va_list ap;
    va_start(ap, pParent);
    char *pMsg = sprintfMessage(msg, ap);
    va_end(ap);
    
    int result = messageQuestionCore(pMsg, modal, buttonType, 0, pParent);
    
    free(pMsg);
    
    return result;
}

int MessageDialog::messageQuestion(const Glib::ustring& msg, bool modal,
	Gtk::ButtonsType buttonType, int response, ...)
{
    va_list ap;
    va_start(ap, response);
    char *pMsg = sprintfMessage(msg, ap);
    va_end(ap);
    
    int result = messageQuestionCore(pMsg, modal, buttonType, response, NULL);
    
    free(pMsg);
    
    return result;
}

int MessageDialog::messageQuestion(const Glib::ustring& msg, bool modal,
	Gtk::ButtonsType buttonType, int response, Gtk::Window *pParent, ...)
{
    va_list ap;
    va_start(ap, pParent);
    char *pMsg = sprintfMessage(msg, ap);
    va_end(ap);
    
    int result = messageQuestionCore(pMsg, modal, buttonType, response, pParent);
    
    free(pMsg);
    
    return result;
}

int MessageDialog::messageQuestionCore(const Glib::ustring& msg, bool modal,
	Gtk::ButtonsType buttonType, int response, Gtk::Window *pParent)
{
    Gtk::MessageDialog dlg(msg, true, Gtk::MESSAGE_QUESTION, buttonType, modal);
    dlg.set_title(_("Question"));
    if (response)
    	dlg.set_default_response(response);
    if (pParent) {
	    dlg.set_transient_for(*pParent);
	    dlg.set_position(Gtk::WIN_POS_CENTER_ON_PARENT);
    }
    else
	    dlg.set_position(Gtk::WIN_POS_CENTER);

    return dlg.run();
}

//_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
// Notify message
Gtk::NotifyDialog* MessageDialog::createNotifyDialog(const Glib::ustring& msg,
	Gtk::Window *pParent, va_list ap)
{
    char *pMsg = sprintfMessage(msg, ap);

    Gtk::NotifyDialog *pDialog =
    	new Gtk::NotifyDialog(pMsg, true, Gtk::MESSAGE_INFO, Gtk::BUTTONS_NONE, true);
    free(pMsg);

    if (!pDialog)
    	return NULL;

    pDialog->set_title(_("Notify"));
    if (pParent) {
	    pDialog->set_transient_for(*pParent);
	    pDialog->set_position(Gtk::WIN_POS_CENTER_ON_PARENT);
    }
    else
	    pDialog->set_position(Gtk::WIN_POS_CENTER);
	
	return pDialog;
}

// void interface
void MessageDialog::messageNotify(const sigc::slot<void> &slot,
	const Glib::ustring& msg, ...)
{
    va_list ap;
    va_start(ap, msg);
	Gtk::NotifyDialog* pDialog = createNotifyDialog(msg, NULL, ap);
    va_end(ap);
    
    messageNotifyCore(slot, pDialog, true);
}

void MessageDialog::messageNotify(const sigc::slot<void> &slot,
	const Glib::ustring& msg, Gtk::Window *pParent, ...)
{
    va_list ap;
    va_start(ap, pParent);
	Gtk::NotifyDialog* pDialog = createNotifyDialog(msg, pParent, ap);
    va_end(ap);
    
    messageNotifyCore(slot, pDialog, true);
}

// bool interface
bool MessageDialog::messageNotify(const sigc::slot<bool> &slot,
	const Glib::ustring& msg, ...)
{
    va_list ap;
    va_start(ap, msg);
	Gtk::NotifyDialog* pDialog = createNotifyDialog(msg, NULL, ap);
    va_end(ap);
    
    return messageNotifyCore(slot, pDialog, true);
}

bool MessageDialog::messageNotify(const sigc::slot<bool> &slot,
	const Glib::ustring& msg, Gtk::Window *pParent, ...)
{
    va_list ap;
    va_start(ap, pParent);
	Gtk::NotifyDialog* pDialog = createNotifyDialog(msg, pParent, ap);
    va_end(ap);
    
    return messageNotifyCore(slot, pDialog, true);
}

// int interface
int MessageDialog::messageNotify(const sigc::slot<int> &slot,
	const Glib::ustring& msg, ...)
{
    va_list ap;
    va_start(ap, msg);
	Gtk::NotifyDialog* pDialog = createNotifyDialog(msg, NULL, ap);
    va_end(ap);
    
    return messageNotifyCore(slot, pDialog, true);
}

int MessageDialog::messageNotify(const sigc::slot<int> &slot,
	const Glib::ustring& msg, Gtk::Window *pParent, ...)
{
    va_list ap;
    va_start(ap, pParent);
	Gtk::NotifyDialog* pDialog = createNotifyDialog(msg, pParent, ap);
    va_end(ap);
    
    return messageNotifyCore(slot, pDialog, true);
}

// pointer interface
void* MessageDialog::messageNotify(const sigc::slot<void*> &slot,
	const Glib::ustring& msg, ...)
{
    va_list ap;
    va_start(ap, msg);
	Gtk::NotifyDialog* pDialog = createNotifyDialog(msg, NULL, ap);
    va_end(ap);
    
    return messageNotifyCore(slot, pDialog, true);
}

void* MessageDialog::messageNotify(const sigc::slot<void*> &slot,
	const Glib::ustring& msg, Gtk::Window *pParent, ...)
{
    va_list ap;
    va_start(ap, pParent);
	Gtk::NotifyDialog* pDialog = createNotifyDialog(msg, pParent, ap);
    va_end(ap);
    
    return messageNotifyCore(slot, pDialog, true);
}

// message notify with user dialog
void MessageDialog::messageNotifyWithUserDialog(const sigc::slot<void> &slot,
	Gtk::Dialog &dlg)
{
    messageNotifyCore(slot, &dlg, false);
}

bool MessageDialog::messageNotifyWithUserDialog(const sigc::slot<bool> &slot,
	Gtk::Dialog &dlg)
{
    return messageNotifyCore(slot, &dlg, false);
}

int MessageDialog::messageNotifyWithUserDialog(const sigc::slot<int> &slot,
	Gtk::Dialog &dlg)
{
    return messageNotifyCore(slot, &dlg, false);
}

void* MessageDialog::messageNotifyWithUserDialog(const sigc::slot<void*> &slot,
	Gtk::Dialog &dlg)
{
    return messageNotifyCore(slot, &dlg, false);
}

// message notify main processes
void MessageDialog::messageNotifyCore(const sigc::slot<void> &slot,
	Gtk::Dialog* pDialog, bool free)
{
    bool end = false;
    Glib::Thread *pthread;

	if (pDialog)
		pDialog->show();

    try {
    	const sigc::slot<void> notifySlot = sigc::bind
    		(sigc::ptr_fun(&notifyVoidFunc), slot, &end);
	    pthread = Glib::Thread::create(notifySlot, true);
    }
    catch (Glib::ThreadError error) {
		fprintf(stderr, "%s: %s\n", APP_NAME, error.what().c_str());
		if (pDialog)
			delete pDialog;
		return;
    }

	waitNotifyFunc(end);
	
	pthread->join();

	if (pDialog && free) {
	    pDialog->hide();
	    delete pDialog;
	}
}

bool MessageDialog::messageNotifyCore(const sigc::slot<bool> &slot,
	Gtk::Dialog* pDialog, bool free)
{
	bool ret = false;
    bool end = false;
    Glib::Thread *pthread;

	if (pDialog)
		pDialog->show();

    try {
    	const sigc::slot<void> notifySlot = sigc::bind
    		(sigc::ptr_fun(&notifyBoolFunc), slot, &end, &ret);
	    pthread = Glib::Thread::create(notifySlot, true);
    }
    catch (Glib::ThreadError error) {
		fprintf(stderr, "%s: %s\n", APP_NAME, error.what().c_str());
		if (pDialog)
			delete pDialog;
		return false;
    }

	waitNotifyFunc(end);

	pthread->join();

	if (pDialog && free) {
	    pDialog->hide();
	    delete pDialog;
	}
	
	return ret;
}

int MessageDialog::messageNotifyCore(const sigc::slot<int> &slot,
	Gtk::Dialog* pDialog, bool free)
{
	int ret = 0;
    bool end = false;
    Glib::Thread *pthread;

	if (pDialog)
		pDialog->show();

    try {
    	const sigc::slot<void> notifySlot = sigc::bind
    		(sigc::ptr_fun(&notifyIntFunc), slot, &end, &ret);
	    pthread = Glib::Thread::create(notifySlot, true);
    }
    catch (Glib::ThreadError error) {
		fprintf(stderr, "%s: %s\n", APP_NAME, error.what().c_str());
		if (pDialog)
			delete pDialog;
		return 0;
    }

	waitNotifyFunc(end);

	pthread->join();

	if (pDialog && free) {
	    pDialog->hide();
	    delete pDialog;
	}
	
	return ret;
}

void* MessageDialog::messageNotifyCore(const sigc::slot<void*> &slot,
	Gtk::Dialog* pDialog, bool free)
{
	void *ret = NULL;
    bool end = false;
    Glib::Thread *pthread;

	if (pDialog)
		pDialog->show();

    try {
    	const sigc::slot<void> notifySlot = sigc::bind
    		(sigc::ptr_fun(&notifyPointerFunc), slot, &end, &ret);
	    pthread = Glib::Thread::create(notifySlot, true);
    }
    catch (Glib::ThreadError error) {
		fprintf(stderr, "%s: %s\n", APP_NAME, error.what().c_str());
		if (pDialog)
			delete pDialog;
		return NULL;
    }

	waitNotifyFunc(end);

	pthread->join();

	if (pDialog && free) {
	    pDialog->hide();
	    delete pDialog;
	}
	
	return ret;
}
