/**
 * $Id: messagedialog.h,v 1.1 2007/03/04 12:53:16 ozawa Exp $
 * 
 * Copyright (C) 2007- ONGS Inc. All rights reserved.
 * 
 * author: Masanori OZAWA (ozawa@ongs.co.jp)
 * version: $Revision: 1.1 $
 */

#ifndef ___MESSAGEDIALOG_H
#define ___MESSAGEDIALOG_H

#include "gtkmm.h"

#define msgError(x)                 MessageDialog::messageError(x)
#define msgWarning(x)               MessageDialog::messageWarning(x)
#define msgInfo(x)                  MessageDialog::messageInfo(x)
#define msgQuestion(x, b)           MessageDialog::messageQuestion(x, b)
#define msgNotify(slot, x)          MessageDialog::messageNotify(slot, x)

#define msgfError(x, ...)           MessageDialog::messageError(x, __VA_ARGS__)
#define msgfWarning(x, ...)         MessageDialog::messageWarning(x, __VA_ARGS__)
#define msgfInfo(x, ...)            MessageDialog::messageInfo(x, __VA_ARGS__)
#define msgfQuestion(x, ...)        MessageDialog::messageQuestion(x, __VA_ARGS__)
#define msgfNotify(slot, x, ...)    MessageDialog::messageNotify(slot, x, __VA_ARGS__)

namespace Gtk {
class NotifyDialog;
};	/* namespace Gtk */

class MessageDialog {
public:
	virtual ~MessageDialog();

private:
    MessageDialog();
    
public:
    static void messageError(const Glib::ustring& msg, bool modal = true, ...);
    static void messageError(const Glib::ustring& msg, bool modal, Gtk::Window *pParent, ...);

    static void messageWarning(const Glib::ustring& msg, bool modal = true, ...);
    static void messageWarning(const Glib::ustring& msg, bool modal, Gtk::Window *pParent, ...);

    static void messageInfo(const Glib::ustring& msg, bool modal = true, ...);
    static void messageInfo(const Glib::ustring& msg, bool modal, Gtk::Window *pParent, ...);

    static int messageQuestion(const Glib::ustring& msg, bool modal = true, Gtk::ButtonsType buttonType = Gtk::BUTTONS_OK_CANCEL, ...);
    static int messageQuestion(const Glib::ustring& msg, bool modal, Gtk::ButtonsType buttonType, Gtk::Window *pParent, ...);
    static int messageQuestion(const Glib::ustring& msg, bool modal, Gtk::ButtonsType buttonType, int response, ...);
    static int messageQuestion(const Glib::ustring& msg, bool modal, Gtk::ButtonsType buttonType, int response, Gtk::Window *pParent, ...);

    static void messageNotify(const sigc::slot<void> &slot, const Glib::ustring& msg, ...);
    static bool messageNotify(const sigc::slot<bool> &slot, const Glib::ustring& msg, ...);
    static int  messageNotify(const sigc::slot<int> &slot, const Glib::ustring& msg, ...);
    static void* messageNotify(const sigc::slot<void*> &slot, const Glib::ustring& msg, ...);

    static void messageNotify(const sigc::slot<void> &slot, const Glib::ustring& msg, Gtk::Window *pParent, ...);
    static bool messageNotify(const sigc::slot<bool> &slot, const Glib::ustring& msg, Gtk::Window *pParent, ...);
    static int  messageNotify(const sigc::slot<int> &slot, const Glib::ustring& msg, Gtk::Window *pParent, ...);
    static void* messageNotify(const sigc::slot<void*> &slot, const Glib::ustring& msg, Gtk::Window *pParent, ...);

	static void messageNotifyWithUserDialog(const sigc::slot<void> &slot, Gtk::Dialog &dlg);
	static bool messageNotifyWithUserDialog(const sigc::slot<bool> &slot, Gtk::Dialog &dlg);
	static int messageNotifyWithUserDialog(const sigc::slot<int> &slot, Gtk::Dialog &dlg);
	static void* messageNotifyWithUserDialog(const sigc::slot<void*> &slot, Gtk::Dialog &dlg);

private:
    static void messageErrorCore(const Glib::ustring& msg, bool modal, Gtk::Window *pParent);

    static void messageWarningCore(const Glib::ustring& msg, bool modal, Gtk::Window *pParent);

    static void messageInfoCore(const Glib::ustring& msg, bool modal, Gtk::Window *pParent);

    static int messageQuestionCore(const Glib::ustring& msg, bool modal, Gtk::ButtonsType buttonType, int response, Gtk::Window *pParent);

    static void messageNotifyCore(const sigc::slot<void> &slot, Gtk::Dialog* pDialog, bool free);
    static bool messageNotifyCore(const sigc::slot<bool> &slot, Gtk::Dialog* pDialog, bool free);
    static int  messageNotifyCore(const sigc::slot<int> &slot, Gtk::Dialog* pDialog, bool free);
    static void* messageNotifyCore(const sigc::slot<void*> &slot, Gtk::Dialog* pDialog, bool free);

	static Gtk::NotifyDialog* createNotifyDialog(const Glib::ustring& msg, Gtk::Window *pParent, va_list ap);
};

#endif // MESSAGEDIALOG_H
