/*
 * $Id: senddialog.cpp,v 1.8 2007/03/04 12:53:16 ozawa Exp $
 *
 * Copyright 2003- ONGS Inc. All rights reserved.
 * 
 * author: Masanori OZAWA (ozawa@ongs.co.jp)
 * version: $Revision: 1.8 $
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY ONGS INC ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL ONGS INC OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * The views and conclusions contained in the software and documentation are
 * those of the authors and should not be interpreted as representing official
 * policies, either expressed or implied, of the ONGS Inc.
 * 
 */

#include "../include/fusenshi.h"
#include "wndservice.h"
#include "senddialog.h"
#include "messagedialog.h"

#include <netdb.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <sys/types.h>

static guint onUpdateServerlist(gpointer data);

/**
 * 󥹥ȥ饯
 */
SendDialog::SendDialog()
{
    set_title(_("Fusenshi - Send"));
    set_wmclass(APP_NAME, "SendDialog");
    set_modal(true);
    set_size_request(580, 300);
    property_destroy_with_parent() = true;

    m_nClients = 0;

    // ĥ꡼ӥ塼ι
    m_cTreeView.get_selection()->set_mode(Gtk::SELECTION_MULTIPLE);

    m_cListStore = Gtk::ListStore::create(m_cColumns);
    m_cTreeView.set_model(m_cListStore);
    m_cTreeView.append_column(_("UserName"), m_cColumns.USERNAME);
    m_cTreeView.append_column(_("HostName"), m_cColumns.HOSTNAME);
    m_cTreeView.append_column(_("IPAddress"), m_cColumns.IPADDR);
    m_cTreeView.append_column(_("SSL"), m_cColumns.SSL);

    int fixedWidth[] = {100, 185, 110, 30};
    Glib::ListHandle<Gtk::TreeViewColumn*> columns = m_cTreeView.get_columns();
    Glib::ListHandle<Gtk::TreeViewColumn*>::iterator iter = columns.begin();
    Glib::ListHandle<Gtk::TreeViewColumn*>::iterator end = columns.end();
    
    for (int count = 0; iter != end; iter++, count++) {
        (*iter)->set_sizing(Gtk::TREE_VIEW_COLUMN_FIXED);
        (*iter)->set_fixed_width(fixedWidth[count]);
        (*iter)->set_resizable(true);
    }
    
    //m_cListStore->set_sort_column(m_Columns.USERNAME, Gtk::SORT_ASCENDING);

    // ɥι
    Gtk::HBox *pMainHBox = manage(new Gtk::HBox());
    get_vbox()->pack_start(*pMainHBox, true, true, 3);

	Gtk::VBox *pLeftVBox = manage(new Gtk::VBox());
	Gtk::VBox *pRightVBox = manage(new Gtk::VBox());
	pMainHBox->pack_start(*pLeftVBox, true, true, 3);
	pMainHBox->pack_start(*pRightVBox, false, false, 3);

    Gtk::Label* pLabel = manage(new Gtk::Label(_("Send To:")));
    pLabel->set_justify(Gtk::JUSTIFY_LEFT);
    pLabel->set_alignment(Gtk::ALIGN_LEFT, Gtk::ALIGN_CENTER);
    pLeftVBox->pack_start(*pLabel, false, true, 3);

    Gtk::ScrolledWindow* pScrolledWnd = manage(new Gtk::ScrolledWindow());
    pLeftVBox->pack_start(*pScrolledWnd, true, true, 3);
    pScrolledWnd->add(m_cTreeView);
    
    Gtk::HBox *pHBox = manage(new Gtk::HBox());
    pLabel = manage(new Gtk::Label(_("IP or Hostname:")));
    pLabel->set_justify(Gtk::JUSTIFY_LEFT);
    pLabel->set_alignment(Gtk::ALIGN_LEFT, Gtk::ALIGN_CENTER);
    pHBox->pack_start(*pLabel, false, true, 5);
    pHBox->pack_start(m_cAddressEntry, true, true, 5);
    pLeftVBox->pack_start(*pHBox, false, true, 3);

	// 򥯥ꥢܥ
	Gtk::VButtonBox *pVBBox = manage(new Gtk::VButtonBox(Gtk::BUTTONBOX_END, 5));
	pRightVBox->pack_start(*pVBBox, true, true, 0);
    Gtk::Button* pButton = manage(new Gtk::Button(Gtk::Stock::CLEAR));
    pVBBox->pack_start(*pButton, true, true, 0);
    pButton->set_size_request(100, 32);

    pButton->signal_clicked().connect
        (sigc::mem_fun(*this, &SendDialog::on_clear_clicked));

	// ܥ
    pButton = manage(new Gtk::Button());
    pVBBox->pack_end(*pButton, true, true, 0);
    pButton->set_size_request(100, 32);

    Gtk::Alignment *pAlignment = manage(new Gtk::Alignment(0.5, 0.5, 0, 0));
    pButton->add(*pAlignment);

    pHBox = manage(new Gtk::HBox(false, 2));
    pAlignment->add(*pHBox);

    Gtk::Image *pImage = manage(new Gtk::Image(Gtk::Stock::REFRESH, Gtk::ICON_SIZE_BUTTON));
    pHBox->pack_start(*pImage, false, false, 5);

    pLabel = manage(new Gtk::Label(_("Update")));
    pHBox->pack_end(*pLabel, false, false, 0);
    pLabel->set_justify(Gtk::JUSTIFY_LEFT);

    pButton->signal_clicked().connect
        (sigc::mem_fun(*this, &SendDialog::on_refresh_clicked));

	// 󥻥ܥ
    get_action_area()->set_layout(Gtk::BUTTONBOX_END);

    pButton = manage(new Gtk::Button(Gtk::Stock::CANCEL));
    add_action_widget(*pButton, Gtk::RESPONSE_CANCEL);
    pButton->set_flags(Gtk::CAN_DEFAULT);

	// ܥ
    pButton = manage(new Gtk::Button());
    add_action_widget(*pButton, Gtk::RESPONSE_OK);
    //pButton->set_flags(Gtk::CAN_DEFAULT);

    pAlignment = manage(new Gtk::Alignment(0.5, 0.5, 0, 0));
    pButton->add(*pAlignment);

    pHBox = manage(new Gtk::HBox(false, 2));
    pAlignment->add(*pHBox);

    pImage = manage(new Gtk::Image(Gtk::Stock::OK, Gtk::ICON_SIZE_BUTTON));
    pHBox->pack_start(*pImage, false, false, 5);

    pLabel = manage(new Gtk::Label(_("Send")));
    pHBox->pack_end(*pLabel, false, false, 0);
    pLabel->set_justify(Gtk::JUSTIFY_LEFT);

    set_default_response(Gtk::RESPONSE_CANCEL);
	signal_response().connect
		(mem_fun(*this, &SendDialog::on_dialog_response), false);    

    show_all_children();

    // ꥯȤ
    on_refresh_clicked();
    
    // 䵻極СɽѤΥޡư
    m_nTimer = g_timeout_add
       (500 /* 500 msec */, (GtkFunction)onUpdateServerlist, this);
}

/**
 * ǥȥ饯
 */
SendDialog::~SendDialog()
{
    g_source_remove(m_nTimer);
}

/**
 * ꥹȥǥŪ˼ޤ
 */
Glib::RefPtr<Gtk::ListStore> SendDialog::getListStore()
{
    return m_cListStore;
}

/**
 * 䵻極аޤ
 * 
 * @return 䵻極а
 */
ServerList& SendDialog::getServerList()
{
    return m_cServerList;
}

/**
 * ľ襢ɥ쥹Ϥɲäޤ
 */
bool SendDialog::parseAddressEntry()
{
	Glib::ustring hostname, ip, portstr;
	Glib::ustring str = m_cAddressEntry.get_text();
	Glib::ustring::size_type pos;
	in_port_t port = FUSENSHI_PORT;
	struct in_addr in = {0};
    struct hostent *phent = NULL;

	str = strTrim(str);
	if (str.empty())
		return true;

	pos = str.find(':');
	if (Glib::ustring::npos != pos) {
		portstr = str.substr(pos);
		port = 0;
		if (0 < portstr.length() || 5 >= portstr.length())
			port = atoi(portstr.substr(1).c_str());
		if (!port) {
	    	Glib::ustring msg = _("Unable to parse the port number");
	    	msg += " '" + portstr + "'.";
	    	msgfWarning(msg, this);

	        signal_response().emission_stop();
	        
	        m_cAddressEntry.grab_focus();
	    	return false;
		}
		str = str.substr(0, pos);
	}

    if (ascii2addr(AF_INET, str.c_str(), (void*)&in) != sizeof(in) &&
    	(phent = gethostbyname(str.c_str())) == NULL)
    {
    	Glib::ustring msg = _("Unable to parse the address");
    	msg += " '" + str + "'.";
    	msgfWarning(msg, this);

        signal_response().emission_stop();
        
        m_cAddressEntry.grab_focus();
    	return false;
    }
    
    if (0 != in.s_addr && NULL == phent) {
    	ip = str;
    }
    else if (NULL != phent) {
    	hostname = phent->h_name;
    	ip = addr2ascii(AF_INET, (void*)(phent->h_addr), sizeof(in_addr), NULL);
    }
    else
    	return false; /* not reachable */

	if (!ip.empty())
	    m_cServerList.add("", hostname, ip, port, FNET_EXT_SSL);

	return true;
}

/**
 * 쥹ݥ
 */
void SendDialog::on_dialog_response(int nResponse)
{
    if (Gtk::RESPONSE_OK != nResponse)
    	return;

    m_cServerList.clear();
    
	// ľ襢ɥ쥹β
    if (!parseAddressEntry())
    	return;

    // 򤷤Ƥ
	Glib::RefPtr<Gtk::TreeSelection> selection = m_cTreeView.get_selection();
	
	if (selection)
	    selection->selected_foreach
    	    (sigc::mem_fun2(*this, &SendDialog::on_selected_foreach));

	// 褬򤵤줿å
	if (0 >= m_cServerList.size()) {
		msgfInfo(_("Please select the target machine to send."), this);

        signal_response().emission_stop();

        m_cTreeView.grab_focus();
    	return;
	}
}

/**
 * ꥢܥ󲡲
 */
void SendDialog::on_clear_clicked()
{
	Glib::RefPtr<Gtk::TreeSelection> selection = m_cTreeView.get_selection();
	
	if (selection)
		selection->unselect_all();
}

/**
 * ܥ󲡲
 */
void SendDialog::on_refresh_clicked()
{
    // 
    FusenshiServerInfo* pServerInfo = FusenshiServerInfo::getInstance();
    
    if (pServerInfo) {
        pServerInfo->getServerList().clear();
    }
    getListStore()->clear();
    m_nClients = 0;
    
    // ꥯȤ
    FusenshiClient::sendServerInfoRequest(PropertyData::get());
}

/**
 * 򤷤ޤ
 */
void SendDialog::on_selected_foreach
    (const Gtk::TreeModel::Path& path, const Gtk::TreeModel::iterator& iter)
{
    m_cServerList.add((*iter)[m_cColumns.USERNAME],
                      (*iter)[m_cColumns.HOSTNAME],
                      (*iter)[m_cColumns.IPADDR],
                      (*iter)[m_cColumns.PORT],
                      (*iter)[m_cColumns.EXTINFO]);
}

/**
 * 䵻極Сιޡ
 */
static guint onUpdateServerlist(gpointer data)
{
    SendDialog *pDialog = (SendDialog*)data;

    if (NULL == pDialog || !pDialog->is_mapped()) {
        return FALSE; // Stop Timer
    }

    // ĥ꡼
    FusenshiServerInfo* pServerInfo = FusenshiServerInfo::getInstance();
    
    if (pServerInfo) {
        ServerList& serverList = pServerInfo->getServerList();
        
        serverList.lockList();
        
        std::list<Glib::ustring>& username = serverList.getUserNameList();
        std::list<Glib::ustring>& hostname = serverList.getHostNameList();
        std::list<Glib::ustring>& ipaddr = serverList.getIPAddrList();
        std::list<in_port_t>& port = serverList.getPortList();
        std::list<u_int32_t>& extinfo = serverList.getExtInfoList();
        
        std::list<Glib::ustring>::iterator uiter;
        std::list<Glib::ustring>::iterator hiter;
        std::list<Glib::ustring>::iterator iiter;
        std::list<in_port_t>::iterator     piter;
        std::list<u_int32_t>::iterator     eiter;
        
        uiter = username.begin();
        hiter = hostname.begin();
        iiter = ipaddr.begin();
        piter = port.begin();
        eiter = extinfo.begin();
        
        for (int count = 0; iiter != ipaddr.end();
             uiter++, hiter++, iiter++, piter++, eiter++, count++)
        {
            if (count < pDialog->m_nClients) continue;

            // ɲä
            Gtk::TreeIter treeiter = pDialog->getListStore()->append();
            treeiter->set_value(pDialog->m_cColumns.USERNAME, *uiter);
            treeiter->set_value(pDialog->m_cColumns.HOSTNAME, *hiter);
            treeiter->set_value(pDialog->m_cColumns.IPADDR, *iiter);
            treeiter->set_value(pDialog->m_cColumns.SSL, (*eiter & FNET_EXT_SSL ? true : false));
            treeiter->set_value(pDialog->m_cColumns.PORT, *piter);
            treeiter->set_value(pDialog->m_cColumns.EXTINFO, *eiter);

            pDialog->m_nClients++;
        }
        
        serverList.unlockList();
    }
    
    return TRUE;
}
